@@ -19,7 +19,8 @@ import stat
 from datetime import datetime
 from threading import Lock
 
-from sos.utilities import sos_get_command_output, is_executable
+from importlib.util import find_spec
+from sos.utilities import sos_get_command_output
 
 try:
     import selinux
@@ -152,7 +153,7 @@ class FileCacheArchive(Archive):
         return (os.path.join(self._archive_root, name))
 
     def join_sysroot(self, path):
-        if path.startswith(self.sysroot):
+        if not self.sysroot or path.startswith(self.sysroot):
             return path
         if path[0] == os.sep:
             path = path[1:]
@@ -250,7 +251,7 @@ class FileCacheArchive(Archive):
 
         return dest
 
-    def _check_path(self, src, path_type, dest=None, force=False):
+    def check_path(self, src, path_type, dest=None, force=False):
         """Check a new destination path in the archive.
 
             Since it is possible for multiple plugins to collect the same
@@ -325,12 +326,26 @@ class FileCacheArchive(Archive):
             return None
         return dest
 
+    def _copy_attributes(self, src, dest):
+        # copy file attributes, skip SELinux xattrs for /sys and /proc
+        try:
+            stat = os.stat(src)
+            if src.startswith("/sys/") or src.startswith("/proc/"):
+                shutil.copymode(src, dest)
+                os.utime(dest, ns=(stat.st_atime_ns, stat.st_mtime_ns))
+            else:
+                shutil.copystat(src, dest)
+            os.chown(dest, stat.st_uid, stat.st_gid)
+        except Exception as e:
+            self.log_debug("caught '%s' setting attributes of '%s'"
+                           % (e, dest))
+
     def add_file(self, src, dest=None):
         with self._path_lock:
             if not dest:
                 dest = src
 
-            dest = self._check_path(dest, P_FILE)
+            dest = self.check_path(dest, P_FILE)
             if not dest:
                 return
 
@@ -341,26 +356,13 @@ class FileCacheArchive(Archive):
                 try:
                     shutil.copy(src, dest)
                 except OSError as e:
-                    self.log_info("File not collected: '%s'" % e)
-                except IOError as e:
                     # Filter out IO errors on virtual file systems.
                     if src.startswith("/sys/") or src.startswith("/proc/"):
                         pass
                     else:
-                        self.log_info("caught '%s' copying '%s'" % (e, src))
+                        self.log_info("File %s not collected: '%s'" % (src, e))
 
-                # copy file attributes, skip SELinux xattrs for /sys and /proc
-                try:
-                    stat = os.stat(src)
-                    if src.startswith("/sys/") or src.startswith("/proc/"):
-                        shutil.copymode(src, dest)
-                        os.utime(dest, ns=(stat.st_atime_ns, stat.st_mtime_ns))
-                    else:
-                        shutil.copystat(src, dest)
-                    os.chown(dest, stat.st_uid, stat.st_gid)
-                except Exception as e:
-                    self.log_debug("caught '%s' setting attributes of '%s'"
-                                   % (e, dest))
+                self._copy_attributes(src, dest)
                 file_name = "'%s'" % src
             else:
                 # Open file case: first rewind the file to obtain
@@ -382,24 +384,20 @@ class FileCacheArchive(Archive):
             # over any exixting content in the archive, since it is used by
             # the Plugin postprocessing hooks to perform regex substitution
             # on file content.
-            dest = self._check_path(dest, P_FILE, force=True)
+            dest = self.check_path(dest, P_FILE, force=True)
 
             f = codecs.open(dest, mode, encoding='utf-8')
             if isinstance(content, bytes):
                 content = content.decode('utf8', 'ignore')
             f.write(content)
             if os.path.exists(src):
-                try:
-                    shutil.copystat(src, dest)
-                except OSError as e:
-                    self.log_error("Unable to add '%s' to archive: %s" %
-                                   (dest, e))
+                self._copy_attributes(src, dest)
             self.log_debug("added string at '%s' to FileCacheArchive '%s'"
                            % (src, self._archive_root))
 
     def add_binary(self, content, dest):
         with self._path_lock:
-            dest = self._check_path(dest, P_FILE)
+            dest = self.check_path(dest, P_FILE)
             if not dest:
                 return
 
@@ -411,7 +409,7 @@ class FileCacheArchive(Archive):
     def add_link(self, source, link_name):
         self.log_debug("adding symlink at '%s' -> '%s'" % (link_name, source))
         with self._path_lock:
-            dest = self._check_path(link_name, P_LINK)
+            dest = self.check_path(link_name, P_LINK)
             if not dest:
                 return
 
@@ -478,7 +476,6 @@ class FileCacheArchive(Archive):
             else:
                 self.log_debug("No link follow up: source=%s link_name=%s" %
                                (source, link_name))
-        self.log_debug("leaving add_link()")
 
     def add_dir(self, path):
         """Create a directory in the archive.
@@ -487,10 +484,10 @@ class FileCacheArchive(Archive):
         """
         # Establish path structure
         with self._path_lock:
-            self._check_path(path, P_DIR)
+            self.check_path(path, P_DIR)
 
     def add_node(self, path, mode, device):
-        dest = self._check_path(path, P_NODE)
+        dest = self.check_path(path, P_NODE)
         if not dest:
             return
 
@@ -503,7 +500,7 @@ class FileCacheArchive(Archive):
                     self.log_info("add_node: %s - mknod '%s'" % (msg, dest))
                     return
                 raise e
-            shutil.copystat(path, dest)
+            self._copy_attributes(path, dest)
 
     def name_max(self):
         if 'PC_NAME_MAX' in os.pathconf_names:
@@ -561,17 +558,16 @@ class FileCacheArchive(Archive):
     def finalize(self, method):
         self.log_info("finalizing archive '%s' using method '%s'"
                       % (self._archive_root, method))
-        self._build_archive()
+        try:
+            res = self._build_archive(method)
+        except Exception as err:
+            self.log_error("An error occurred compressing the archive: %s"
+                           % err)
+            return self.name()
+
         self.cleanup()
         self.log_info("built archive at '%s' (size=%d)" % (self._archive_name,
                       os.stat(self._archive_name).st_size))
-        self.method = method
-        try:
-            res = self._compress()
-        except Exception as e:
-            exp_msg = "An error occurred compressing the archive: "
-            self.log_error("%s %s" % (exp_msg, e))
-            return self.name()
 
         if self.enc_opts['encrypt']:
             try:
@@ -638,7 +634,9 @@ class TarFileArchive(FileCacheArchive):
         super(TarFileArchive, self).__init__(name, tmpdir, policy, threads,
                                              enc_opts, sysroot, manifest)
         self._suffix = "tar"
-        self._archive_name = os.path.join(tmpdir, self.name())
+        self._archive_name = os.path.join(
+            tmpdir, self.name()  # lgtm [py/init-calls-subclass]
+        )
 
     def set_tarinfo_from_stat(self, tar_info, fstat, mode=None):
         tar_info.mtime = fstat.st_mtime
@@ -683,44 +681,26 @@ class TarFileArchive(FileCacheArchive):
         # the limit of the underlying FileCacheArchive.
         return super(TarFileArchive, self).name_max()
 
-    def _build_archive(self):
-        tar = tarfile.open(self._archive_name, mode="w")
+    def _build_archive(self, method):
+        if method == 'auto':
+            method = 'xz' if find_spec('lzma') is not None else 'gzip'
+        _comp_mode = method.strip('ip')
+        self._archive_name = self._archive_name + ".%s" % _comp_mode
+        # tarfile does not currently have a consistent way to define comnpress
+        # level for both xz and gzip ('preset' for xz, 'compresslevel' for gz)
+        if method == 'gzip':
+            kwargs = {'compresslevel': 6}
+        else:
+            kwargs = {'preset': 3}
+        tar = tarfile.open(self._archive_name, mode="w:%s" % _comp_mode,
+                           **kwargs)
         # we need to pass the absolute path to the archive root but we
         # want the names used in the archive to be relative.
         tar.add(self._archive_root, arcname=os.path.split(self._name)[1],
                 filter=self.copy_permissions_filter)
         tar.close()
+        self._suffix += ".%s" % _comp_mode
+        return self.name()
 
-    def _compress(self):
-        methods = []
-        # Make sure that valid compression commands exist.
-        for method in ['xz', 'gzip']:
-            if is_executable(method):
-                methods.append(method)
-            else:
-                self.log_info("\"%s\" compression method unavailable" % method)
-        if self.method in methods:
-            methods = [self.method]
-
-        exp_msg = "No compression utilities found."
-        last_error = Exception(exp_msg)
-        for cmd in methods:
-            suffix = "." + cmd.replace('ip', '')
-            cmd = self._policy.get_cmd_for_compress_method(cmd, self._threads)
-            try:
-                exec_cmd = "%s %s" % (cmd, self.name())
-                r = sos_get_command_output(exec_cmd, stderr=True, timeout=0)
-
-                if r['status']:
-                    self.log_error(r['output'])
-                    raise Exception("%s exited with %s" % (exec_cmd,
-                                    r['status']))
-
-                self._suffix += suffix
-                return self.name()
-
-            except Exception as e:
-                last_error = e
-        raise last_error
 
 # vim: set et ts=4 sw=4 :
